#ifndef SPLINESSPLINESEXCEPTION_H
#define SPLINESSPLINESEXCEPTION_H 

// ----------------------------------------------------------------------
//
// Splines_exception
//
// Classes embodying the various excceptions thrown by the splines package
//
// ----------------------------------------------------------------------

#include <string>
#include <stdexcept>

namespace splines {

// class Splines_exception : public std::runtime_error {};
class Splines_exception : public std::runtime_error {
public:
  Splines_exception () : runtime_error("Spline usage exception") {}
};


// =================================
// Exceptions thrown by Grid classes 
// =================================

class Grid_order_exception : public Splines_exception {
public:
  double a; 
  double b;
  Grid_order_exception (double aa, double bb) : a(aa), b(bb) {}
};

class Grid_push_back_order_exception : public Grid_order_exception {
public:
  Grid_push_back_order_exception (double p, double u) :
		Grid_order_exception (p,u) {}
  virtual const char* what() const throw(); 
};

class Grid_append_order_exception : public Grid_order_exception {
public:
  Grid_append_order_exception (double p, double u) :
		Grid_order_exception (p,u) {}
  virtual const char* what() const throw(); 
};

class Grid_limits_order_exception : public Grid_order_exception {
public:
  Grid_limits_order_exception (double low, double hi) :
		Grid_order_exception (low,hi) {}
  virtual const char* what() const throw(); 
};

class Grid_dimension_exception : public Splines_exception {
public:
  unsigned int d; 
  unsigned int n;
  Grid_dimension_exception (unsigned int dd, unsigned int nn) : d(dd), n(nn) {}
};

class Grid_face_dimension_exception : public Grid_dimension_exception {
public:
  Grid_face_dimension_exception (unsigned int dd, unsigned int nn) :
		Grid_dimension_exception (dd,nn) {}
  virtual const char* what() const throw(); 
};

// ===================================
// Exceptions thrown by Spline classes 
// ===================================

class Spline_argument_exception : public Splines_exception {
public:
  unsigned int d; 
  unsigned int n;
  Spline_argument_exception (unsigned int dd, unsigned int nn) : d(dd), n(nn) {}
  virtual const char* what() const throw(); 
};

class Spline_table_exception : public Splines_exception {
public:
  unsigned int d; 
  unsigned int v; 
  unsigned int n;
  Spline_table_exception (unsigned int dd, unsigned int vv, unsigned int nn) 
  : d(dd), v(vv), n(nn) {}
  virtual const char* what() const throw(); 
};


} // end of namespace splines

#endif  /* SPLINESSPLINESEXCEPTION_H  */
